@@ -1,19 +1,28 @@
 #!/usr/bin/env python
+# encoding=utf-8
 #
 #  lock_tests.py:  testing versioned properties
 #
 #  Subversion is a tool for revision control.
-#  See http://subversion.tigris.org for more information.
+#  See http://subversion.apache.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2005-2006, 2008 CollabNet.  All rights reserved.
+#    Licensed to the Apache Software Foundation (ASF) under one
+#    or more contributor license agreements.  See the NOTICE file
+#    distributed with this work for additional information
+#    regarding copyright ownership.  The ASF licenses this file
+#    to you under the Apache License, Version 2.0 (the
+#    "License"); you may not use this file except in compliance
+#    with the License.  You may obtain a copy of the License at
 #
-# This software is licensed as described in the file COPYING, which
-# you should have received as part of this distribution.  The terms
-# are also available at http://subversion.tigris.org/license-1.html.
-# If newer versions of this license are posted there, you may use a
-# newer version instead, at your option.
+#      http://www.apache.org/licenses/LICENSE-2.0
 #
+#    Unless required by applicable law or agreed to in writing,
+#    software distributed under the License is distributed on an
+#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#    KIND, either express or implied.  See the License for the
+#    specific language governing permissions and limitations
+#    under the License.
 ######################################################################
 
 # General modules
@@ -23,11 +32,32 @@ import re, os, stat
 import svntest
 
 # (abbreviation)
-Skip = svntest.testcase.Skip
-SkipUnless = svntest.testcase.SkipUnless
-XFail = svntest.testcase.XFail
+Skip = svntest.testcase.Skip_deco
+SkipUnless = svntest.testcase.SkipUnless_deco
+XFail = svntest.testcase.XFail_deco
+Issues = svntest.testcase.Issues_deco
+Issue = svntest.testcase.Issue_deco
+Wimp = svntest.testcase.Wimp_deco
 Item = svntest.wc.StateItem
 
+######################################################################
+# Helpers
+
+def check_writability(path, writable):
+  bits = stat.S_IWGRP | stat.S_IWOTH | stat.S_IWRITE
+  mode = os.stat(path)[0]
+  if bool(mode & bits) != writable:
+    raise svntest.Failure("path '%s' is unexpectedly %s (mode %o)"
+                          % (path, ["writable", "read-only"][writable], mode))
+
+def is_writable(path):
+  "Raise if PATH is not writable."
+  check_writability(path, True)
+
+def is_readonly(path):
+  "Raise if PATH is not readonly."
+  check_writability(path, False)
+
 ######################################################################
 # Tests
 
@@ -60,7 +90,8 @@ def lock_file(sbox):
                                      '-m', '', file_path)
 
   # --- Meanwhile, in our other working copy... ---
-  err_re = "((.*User jconstant does not own lock on path.*)|(.*423 Locked.*))"
+  err_re = "(svn\: E195022\: File '.*iota' is locked in another)|" + \
+           "(svn\: E160039: User '?jconstant'? does not own lock on path.*iota')"
 
   svntest.main.run_svn(None, 'update', wc_b)
   # -- Try to change a file --
@@ -80,8 +111,8 @@ def lock_file(sbox):
   # change the locked file's properties
   svntest.main.run_svn(None, 'propset', 'sneakyuser', 'Sally', file_path_b)
 
-  err_re = "((.*User jconstant does not own lock on path.*)" + \
-             "|(.*At least one property change failed.*))"
+  err_re = "(svn\: E195022\: File '.*iota' is locked in another)|" + \
+           "(svn\: E160039\: User '?jconstant'? does not own lock on path)"
 
   # attempt (and fail) to commit as user Sally
   svntest.actions.run_and_verify_commit(wc_b, None, None, err_re,
@@ -330,17 +361,11 @@ def enforce_lock(sbox):
   lambda_path = os.path.join(wc_dir, 'A', 'B', 'lambda')
   mu_path = os.path.join(wc_dir, 'A', 'mu')
 
-  # Set some binary properties.
-  propval_path = os.path.join(wc_dir, 'propval.tmp')
-
   # svn:needs-lock value should be forced to a '*'
-  svntest.actions.set_prop(None, 'svn:needs-lock', 'foo', iota_path,
-                           propval_path)
-  svntest.actions.set_prop(None, 'svn:needs-lock', '*', lambda_path,
-                           propval_path)
-  expected_err = ".*svn: warning: To turn off the svn:needs-lock property,.*"
-  svntest.actions.set_prop(expected_err, 'svn:needs-lock', '      ',
-                           mu_path, propval_path)
+  svntest.actions.set_prop('svn:needs-lock', 'foo', iota_path)
+  svntest.actions.set_prop('svn:needs-lock', '*', lambda_path)
+  expected_err = ".*svn: warning: W125005: To turn off the svn:needs-lock property,.*"
+  svntest.actions.set_prop('svn:needs-lock', '      ', mu_path, expected_err)
 
   # Check svn:needs-lock
   svntest.actions.check_prop('svn:needs-lock', iota_path, ['*'])
@@ -388,6 +413,7 @@ def enforce_lock(sbox):
 # Test that updating a file with the "svn:needs-lock" property works,
 # especially on Windows, where renaming A to B fails if B already
 # exists and has its read-only bit set.  See also issue #2278.
+@Issue(2278)
 def update_while_needing_lock(sbox):
   "update handles svn:needs-lock correctly"
 
@@ -674,7 +700,7 @@ def lock_non_existent_file(sbox):
   exit_code, output, error = svntest.main.run_svn(1, 'lock',
                                                   '-m', '', file_path)
 
-  error_msg = "foo' is not under version control"
+  error_msg = "The node '%s' was not found." % os.path.abspath(file_path)
   for line in error:
     if line.find(error_msg) != -1:
       break
@@ -843,9 +869,11 @@ def lock_switched_files(sbox):
   alpha_URL = sbox.repo_url + '/A/B/E/alpha'
 
   svntest.actions.run_and_verify_svn(None, None, [], 'switch',
-                                     iota_URL, gamma_path)
+                                     iota_URL, gamma_path,
+                                     '--ignore-ancestry')
   svntest.actions.run_and_verify_svn(None, None, [], 'switch',
-                                     alpha_URL, lambda_path)
+                                     alpha_URL, lambda_path,
+                                     '--ignore-ancestry')
 
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.tweak('A/D/gamma', 'A/B/lambda', switched='S')
@@ -856,7 +884,15 @@ def lock_switched_files(sbox):
                                      gamma_path, lambda_path)
 
   expected_status.tweak('A/D/gamma', 'A/B/lambda', writelocked='K')
-  expected_status.tweak('A/B/E/alpha', 'iota', writelocked='O')
+
+  # In WC-NG locks are kept per working copy, not per file
+  if svntest.main.wc_is_singledb(wc_dir):
+    # In single-db you see these files are locked locally
+    expected_status.tweak('A/B/E/alpha', 'iota', writelocked='K')
+  else:
+    # In multi-db you see these files are not locked in the right dir
+    expected_status.tweak('A/B/E/alpha', 'iota', writelocked='O')
+
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
   svntest.actions.run_and_verify_svn(None, ".*unlocked", [], 'unlock',
@@ -926,7 +962,8 @@ def lock_uri_encoded(sbox):
 
 #----------------------------------------------------------------------
 # A regression test for a bug when svn:needs-lock and svn:executable
-# interact badly. The bug was fixed in trunk @ r14859.
+# interact badly. The bug was fixed in trunk @ r854933.
+@SkipUnless(svntest.main.is_posix_os)
 def lock_and_exebit1(sbox):
   "svn:needs-lock and svn:executable, part I"
 
@@ -939,11 +976,11 @@ def lock_and_exebit1(sbox):
 
   gamma_path = os.path.join(wc_dir, 'A', 'D', 'gamma')
 
-  expected_err = ".*svn: warning: To turn off the svn:needs-lock property,.*"
+  expected_err = ".*svn: warning: W125005: To turn off the svn:needs-lock property,.*"
   svntest.actions.run_and_verify_svn2(None, None, expected_err, 0,
                                       'ps', 'svn:needs-lock', ' ', gamma_path)
 
-  expected_err = ".*svn: warning: To turn off the svn:executable property,.*"
+  expected_err = ".*svn: warning: W125005: To turn off the svn:executable property,.*"
   svntest.actions.run_and_verify_svn2(None, None, expected_err, 0,
                                       'ps', 'svn:executable', ' ', gamma_path)
 
@@ -1003,6 +1040,7 @@ def lock_and_exebit1(sbox):
 
 #----------------------------------------------------------------------
 # A variant of lock_and_exebit1: same test without unlock
+@SkipUnless(svntest.main.is_posix_os)
 def lock_and_exebit2(sbox):
   "svn:needs-lock and svn:executable, part II"
 
@@ -1015,11 +1053,11 @@ def lock_and_exebit2(sbox):
 
   gamma_path = os.path.join(wc_dir, 'A', 'D', 'gamma')
 
-  expected_err = ".*svn: warning: To turn off the svn:needs-lock property,.*"
+  expected_err = ".*svn: warning: W125005: To turn off the svn:needs-lock property,.*"
   svntest.actions.run_and_verify_svn2(None, None, expected_err, 0,
                                       'ps', 'svn:needs-lock', ' ', gamma_path)
 
-  expected_err = ".*svn: warning: To turn off the svn:executable property,.*"
+  expected_err = ".*svn: warning: W125005: To turn off the svn:executable property,.*"
   svntest.actions.run_and_verify_svn2(None, None, expected_err, 0,
                                      'ps', 'svn:executable', ' ', gamma_path)
 
@@ -1120,50 +1158,22 @@ def repos_lock_with_info(sbox):
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
   # Get repository lock token
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
-                                                              'info', file_url)
-  for line in output:
-    if line.find("Lock Token:") != -1:
-      repos_lock_token = line[12:]
-      break
-  else:
-    print("Error: Lock token not found")
-    raise svntest.Failure
+  repos_lock_token \
+    = svntest.actions.run_and_parse_info(file_url)[0]['Lock Token']
 
   # info with revision option
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
-                                                              'info',
-                                                              file_path, '-r1')
-
-  for line in output:
-    if line.find("Lock Token:") != -1:
-      lock_token = line[12:]
-      break
-  else:
-    print("Error: Lock token not found")
-    raise svntest.Failure
-
-  if (repos_lock_token != lock_token):
-    print("Error: expected repository lock information not found.")
-    raise svntest.Failure
+  expected_infos = [
+      { 'Lock Token' : repos_lock_token },
+    ]
+  svntest.actions.run_and_verify_info(expected_infos, file_path, '-r1')
 
   # info with peg revision
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
-                                                              'info',
-                                                              file_path + '@1')
-  for line in output:
-    if line.find("Lock Token:") != -1:
-      lock_token = line[12:]
-      break
-  else:
-    print("Error: Lock token not found")
-    raise svntest.Failure
+  svntest.actions.run_and_verify_info(expected_infos, file_path + '@1')
 
-  if (repos_lock_token != lock_token):
-    print("Error: expected repository lock information not found.")
-    raise svntest.Failure
 
 #----------------------------------------------------------------------
+@Issue(4126)
+@Skip(svntest.main.is_ra_type_dav_serf) # Issue 4126 unpredictable result
 def unlock_already_unlocked_files(sbox):
   "(un)lock set of files, one already (un)locked"
 
@@ -1263,21 +1273,11 @@ def info_moved_path(sbox):
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
   # Get info for old iota at r1. This shouldn't give us any lock info.
-  exit_code, output, errput = svntest.actions.run_and_verify_svn(
-    None, None, [], 'info', fname2, '-r1')
-
-  # Since we want to make sure that there is *no* lock info, to make this
-  # more robust, we also check that the info command actually output some info.
-  got_url = 0
-  for line in output:
-    if line.find("URL:") >= 0:
-      got_url = 1
-    if line.find("Lock Token:") >= 0:
-      print(fname2 + " was reported as locked.")
-      raise svntest.Failure
-  if not got_url:
-    print("Info didn't output an URL.")
-    raise svntest.Failure
+  expected_infos = [
+      { 'URL'           : '.*' ,
+        'Lock Token'    : None },
+    ]
+  svntest.actions.run_and_verify_info(expected_infos, fname2, '-r1')
 
 #----------------------------------------------------------------------
 def ls_url_encoded(sbox):
@@ -1322,6 +1322,7 @@ def ls_url_encoded(sbox):
 
 #----------------------------------------------------------------------
 # Make sure unlocking a path with the wrong lock token fails.
+@Issue(3794)
 def unlock_wrong_token(sbox):
   "verify unlocking with wrong lock token"
 
@@ -1339,11 +1340,11 @@ def unlock_wrong_token(sbox):
   # Steal the lock as the same author, but using an URL to keep the old token
   # in the WC.
   svntest.actions.run_and_verify_svn(None, ".*locked by user", [], 'lock',
-                                     "--force", file_url)
+                                    "--force", file_url)
 
   # Then, unlocking the WC path should fail.
-  # ### The error message returned is actually this, but let's worry about that
-  # ### another day...
+  ### The error message returned is actually this, but let's worry about that
+  ### another day...
   svntest.actions.run_and_verify_svn2(
     None, None, ".*((No lock on path)|(400 Bad Request))", 0,
     'unlock', file_path)
@@ -1382,6 +1383,7 @@ def examine_lock_encoded_recurse(sbox):
                                         svntest.main.wc_author)
 
 # Trying to unlock someone else's lock with --force should fail.
+@Issue(3801)
 def unlocked_lock_of_other_user(sbox):
   "unlock file locked by other user"
 
@@ -1402,7 +1404,7 @@ def unlocked_lock_of_other_user(sbox):
   if sbox.repo_url.startswith("http"):
     expected_err = ".*403 Forbidden.*"
   else:
-    expected_err = "svn: warning: User '%s' is trying to use a lock owned by "\
+    expected_err = "svn: warning: W160039: User '%s' is trying to use a lock owned by "\
                    "'%s'.*" % (svntest.main.wc_author2, svntest.main.wc_author)
   svntest.actions.run_and_verify_svn2(None, [], expected_err, 0,
                                       'unlock',
@@ -1411,6 +1413,338 @@ def unlocked_lock_of_other_user(sbox):
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
 
+#----------------------------------------------------------------------
+def lock_funky_comment_chars(sbox):
+  "lock a file using a comment with xml special chars"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  # lock a file as wc_author
+  fname = 'iota'
+  file_path = os.path.join(sbox.wc_dir, fname)
+
+  svntest.main.file_append(file_path, "This represents a binary file\n")
+  svntest.main.run_svn(None, 'commit',
+                       '-m', '', file_path)
+  svntest.actions.run_and_verify_svn(None, ".*locked by user", [], 'lock',
+                                     '-m', 'lock & load', file_path)
+
+#----------------------------------------------------------------------
+# Check that the svn:needs-lock usage applies to a specific location
+# in a working copy, not to the working copy overall.
+def lock_twice_in_one_wc(sbox):
+  "try to lock a file twice in one working copy"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  mu_path = os.path.join(wc_dir, 'A', 'mu')
+  mu2_path = os.path.join(wc_dir, 'A', 'B', 'mu')
+
+  # Create a needs-lock file
+  svntest.actions.set_prop('svn:needs-lock', '*', mu_path)
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'commit', wc_dir, '-m', '')
+
+  # Mark the file readonly
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'update', wc_dir)
+
+  # Switch a second location for the same file in the same working copy
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'switch', sbox.repo_url + '/A',
+                                     os.path.join(wc_dir, 'A', 'B'),
+                                     '--ignore-ancestry')
+
+  # Lock location 1
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'lock', mu_path, '-m', 'Locked here')
+
+  # Locking in location 2 should fail ### Currently returns exitcode 0
+  svntest.actions.run_and_verify_svn2(None, None, ".*is already locked.*", 0,
+                                      'lock', '-m', '', mu2_path)
+
+  # Change the file anyway
+  os.chmod(mu2_path, 0700)
+  svntest.main.file_append(mu2_path, "Updated text")
+
+  if svntest.main.wc_is_singledb(wc_dir):
+    # Commit will just succeed as the DB owns the lock. It's a user decision
+    # to commit the other target instead of the one originally locked
+    expected_err = []
+  else:
+    # Commit should fail because it is locked in the other location
+    expected_err = '.*(([Nn]o)|(Server)).*[lL]ock.*'
+
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'commit', mu2_path, '-m', '')
+
+#----------------------------------------------------------------------
+# Test for issue #3524 'Locking path via ra_serf which doesn't exist in
+# HEAD triggers assert'
+@Issue(3524)
+def lock_path_not_in_head(sbox):
+  "lock path that does not exist in HEAD"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  D_path      = os.path.join(wc_dir, 'A', 'D')
+  lambda_path = os.path.join(wc_dir, 'A', 'B', 'lambda')
+
+  # Commit deletion of A/D and A/B/lambda as r2, then update the WC
+  # back to r1.  Then attempt to lock some paths that no longer exist
+  # in HEAD.  These should fail gracefully.
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'delete', lambda_path, D_path)
+  svntest.actions.run_and_verify_svn(None, None, [], 'commit',
+                                     '-m', 'Some deletions', wc_dir)
+  svntest.actions.run_and_verify_svn(None, None, [], 'up', '-r1', wc_dir)
+  expected_lock_fail_err_re = "svn: warning: W160042: " \
+  "((Path .* doesn't exist in HEAD revision)" \
+  "|(L(ock|OCK) request (on '.*' )?failed: 405 Method Not Allowed))"
+  # Issue #3524 These lock attemtps were triggering an assert over ra_serf:
+  #
+  # working_copies\lock_tests-37>svn lock A\D
+  # ..\..\..\subversion\libsvn_client\ra.c:275: (apr_err=235000)
+  # svn: In file '..\..\..\subversion\libsvn_ra_serf\util.c' line 1120:
+  #  assertion failed (ctx->status_code)
+  #
+  # working_copies\lock_tests-37>svn lock A\B\lambda
+  # ..\..\..\subversion\libsvn_client\ra.c:275: (apr_err=235000)
+  # svn: In file '..\..\..\subversion\libsvn_ra_serf\util.c' line 1120:
+  #  assertion failed (ctx->status_code)
+  svntest.actions.run_and_verify_svn2(None, None, expected_lock_fail_err_re,
+                                      0, 'lock', D_path)
+  svntest.actions.run_and_verify_svn2(None, None, expected_lock_fail_err_re,
+                                      0, 'lock', lambda_path)
+
+#----------------------------------------------------------------------
+def verify_path_escaping(sbox):
+  "verify escaping of lock paths"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  # Add test paths using two characters that need escaping in a url, but
+  # are within the normal ascii range
+  file1 = os.path.join(wc_dir, 'file #1')
+  file2 = os.path.join(wc_dir, 'file #2')
+  file3 = os.path.join(wc_dir, 'file #3')
+
+  svntest.main.file_write(file1, 'File 1')
+  svntest.main.file_write(file2, 'File 2')
+  svntest.main.file_write(file3, 'File 3')
+
+  svntest.main.run_svn(None, 'add', file1, file2, file3)
+
+  svntest.main.run_svn(None, 'ci', '-m', 'commit', wc_dir)
+
+  svntest.main.run_svn(None, 'lock', '-m', 'lock 1', file1)
+  svntest.main.run_svn(None, 'lock', '-m', 'lock 2', sbox.repo_url + '/file%20%232')
+  svntest.main.run_svn(None, 'lock', '-m', 'lock 3', file3)
+  svntest.main.run_svn(None, 'unlock', sbox.repo_url + '/file%20%233')
+
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
+  expected_status.add(
+    {
+      'file #1'           : Item(status='  ', writelocked='K', wc_rev='2'),
+      'file #2'           : Item(status='  ', writelocked='O', wc_rev='2'),
+      'file #3'           : Item(status='  ', writelocked='B', wc_rev='2')
+    });
+
+  # Make sure the file locking is reported correctly
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+
+#----------------------------------------------------------------------
+# Issue #3674: Replace + propset of locked file fails over DAV
+@XFail(svntest.main.is_ra_type_dav)
+@Issue(3674)
+def replace_and_propset_locked_path(sbox):
+  "test replace + propset of locked file"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  mu_path = os.path.join(wc_dir, 'A', 'mu')
+  G_path = os.path.join(wc_dir, 'A', 'D', 'G')
+  rho_path = os.path.join(G_path, 'rho')
+
+  # Lock mu and A/D/G/rho.
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'lock', mu_path, rho_path,
+                                     '-m', 'Locked')
+
+  # Now replace and propset on mu.
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'rm', '--keep-local', mu_path)
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'add', mu_path)
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'propset', 'foo', 'bar', mu_path)
+
+  # Commit mu.
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'commit', '-m', '', mu_path)
+
+  # Let's try this again where directories are involved, shall we?
+  # Replace A/D/G and A/D/G/rho, propset on A/D/G/rho.
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'rm', G_path)
+  # Recreate path for single-db
+  if not os.path.exists(G_path):
+    os.mkdir(G_path)
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'add', G_path)
+  svntest.main.file_append(rho_path, "This is the new file 'rho'.\n")
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'add', rho_path)
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'propset', 'foo', 'bar', rho_path)
+
+  # And commit G.
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'commit', '-m', '', G_path)
+
+
+#----------------------------------------------------------------------
+def cp_isnt_ro(sbox):
+  "uncommitted svn:needs-lock add/cp not read-only"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  mu_URL = sbox.repo_url + '/A/mu'
+  mu_path = os.path.join(wc_dir, 'A', 'mu')
+  mu2_path = os.path.join(wc_dir, 'A', 'mu2')
+  mu3_path = os.path.join(wc_dir, 'A', 'mu3')
+  kappa_path = os.path.join(wc_dir, 'kappa')
+  open(kappa_path, 'w').write("This is the file 'kappa'.\n")
+
+  ## added file
+  sbox.simple_add('kappa')
+  svntest.actions.set_prop('svn:needs-lock', 'yes', kappa_path)
+  is_writable(kappa_path)
+  sbox.simple_commit('kappa')
+  is_readonly(kappa_path)
+
+  ## versioned file
+  svntest.actions.set_prop('svn:needs-lock', 'yes', mu_path)
+  is_writable(mu_path)
+  sbox.simple_commit('A/mu')
+  is_readonly(mu_path)
+
+  # At this point, mu has 'svn:needs-lock' set
+
+  ## wc->wc copied file
+  svntest.main.run_svn(None, 'copy', mu_path, mu2_path)
+  is_writable(mu2_path)
+  sbox.simple_commit('A/mu2')
+  is_readonly(mu2_path)
+
+  ## URL->wc copied file
+  svntest.main.run_svn(None, 'copy', mu_URL, mu3_path)
+  is_writable(mu3_path)
+  sbox.simple_commit('A/mu3')
+  is_readonly(mu3_path)
+
+
+#----------------------------------------------------------------------
+# Issue #3525: Locked file which is scheduled for delete causes tree
+# conflict
+@Issue(3525)
+def update_locked_deleted(sbox):
+  "updating locked scheduled-for-delete file"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  iota_path = sbox.ospath('iota')
+  mu_path = sbox.ospath('A/mu')
+  alpha_path = sbox.ospath('A/B/E/alpha')
+
+  svntest.main.run_svn(None, 'lock', '-m', 'locked', mu_path, iota_path,
+                       alpha_path)
+  sbox.simple_rm('iota')
+  sbox.simple_rm('A/mu')
+  sbox.simple_rm('A/B/E')
+
+  # Create expected output tree for an update.
+  expected_output = svntest.wc.State(wc_dir, {
+  })
+
+  # Create expected status tree for the update.
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
+  expected_status.tweak('A/B/E', status='D ')
+  expected_status.tweak('iota', 'A/mu', 'A/B/E/alpha',
+                        status='D ', writelocked='K')
+  expected_status.tweak('A/B/E/beta', status='D ')
+
+  svntest.actions.run_and_verify_update(wc_dir, expected_output,
+                                        None, expected_status)
+
+  # Now we steal the lock of iota and A/mu via URL and retry
+  svntest.main.run_svn(None, 'lock', '-m', 'locked', sbox.repo_url + '/iota',
+                       '--force', sbox.repo_url + '/A/mu',
+                       sbox.repo_url + '/A/B/E/alpha')
+
+  expected_status.tweak('iota', 'A/mu', 'A/B/E/alpha',
+                        status='D ', writelocked='O')
+
+  svntest.actions.run_and_verify_update(wc_dir, expected_output,
+                                        None, expected_status)
+
+
+#----------------------------------------------------------------------
+def block_unlock_if_pre_unlock_hook_fails(sbox):
+  "block unlock operation if pre-unlock hook fails"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  repo_dir = sbox.repo_dir
+
+  svntest.actions.create_failing_hook(repo_dir, "pre-unlock", "error text")
+
+  # lock a file.
+  pi_path = os.path.join(wc_dir, 'A', 'D', 'G', 'pi')
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
+  expected_status.tweak('A/D/G/pi', writelocked='K')
+
+  svntest.actions.run_and_verify_svn(None, ".*locked by user", [], 'lock',
+                                     '-m', '', pi_path)
+
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+  # Make sure the unlock operation fails as pre-unlock hook blocks it.
+  expected_unlock_fail_err_re = ".*error text|.*500 Internal Server Error"
+  svntest.actions.run_and_verify_svn2(None, None, expected_unlock_fail_err_re,
+                                      1, 'unlock', pi_path)
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+#----------------------------------------------------------------------
+def lock_invalid_token(sbox):
+  "verify pre-lock hook returning invalid token"
+
+  sbox.build()
+
+  hook_path = os.path.join(sbox.repo_dir, 'hooks', 'pre-lock')
+  svntest.main.create_python_hook_script(hook_path,
+    '# encoding=utf-8\n'
+    'import sys\n'
+    'sys.stdout.write("тест")\n'
+    'sys.exit(0)\n')
+
+  fname = 'iota'
+  file_path = os.path.join(sbox.wc_dir, fname)
+
+  svntest.actions.run_and_verify_svn2(None, None,
+                                      "svn: warning: W160037: " \
+                                      ".*scheme.*'opaquelocktoken'", 0,
+                                      'lock', '-m', '', file_path)
+
+
 ########################################################################
 # Run the tests
 
@@ -1440,17 +1774,25 @@ test_list = [ None,
               lock_several_files,
               lock_switched_files,
               lock_uri_encoded,
-              SkipUnless(lock_and_exebit1, svntest.main.is_posix_os),
-              SkipUnless(lock_and_exebit2, svntest.main.is_posix_os),
+              lock_and_exebit1,
+              lock_and_exebit2,
               commit_xml_unsafe_file_unlock,
               repos_lock_with_info,
               unlock_already_unlocked_files,
               info_moved_path,
               ls_url_encoded,
-              XFail(unlock_wrong_token, svntest.main.is_ra_type_dav),
+              unlock_wrong_token,
               examine_lock_encoded_recurse,
-              XFail(unlocked_lock_of_other_user,
-                    svntest.main.is_ra_type_dav)
+              unlocked_lock_of_other_user,
+              lock_funky_comment_chars,
+              lock_twice_in_one_wc,
+              lock_path_not_in_head,
+              verify_path_escaping,
+              replace_and_propset_locked_path,
+              cp_isnt_ro,
+              update_locked_deleted,
+              block_unlock_if_pre_unlock_hook_fails,
+              lock_invalid_token,
             ]
 
 if __name__ == '__main__':
